/* ========================================
 *
 * Copyright YOUR COMPANY, THE YEAR
 * All Rights Reserved
 * UNPUBLISHED, LICENSED SOFTWARE.
 *
 * CONFIDENTIAL AND PROPRIETARY INFORMATION
 * WHICH IS THE PROPERTY OF your company.
 *
 * ========================================
*/

#include "project.h"
#include "stdio.h"
#include <main.h>
#include "BLE_APP.h"
#include "LiquidCrystal_I2C.h" //Librería LCD

uint32_t Addr = 0x27; //Dirección LCD

//Variables
static int temperature=99; //Temperatura total
static int temperature1=0; //Primer Byte temperatura
static int temperature2=0; //Segundo Byte temperatura
static int humidity=99; //Humedad total
static int humidity1=0; //Primer Byte humedad
static int humidity2=0; //Segundo Byte humedad

CYBLE_GATTS_HANDLE_VALUE_NTF_T notificationHandle;

CYBLE_CONN_HANDLE_T connectionHandle;

uint8 startNotification;
uint8 deviceConnected = 0;

int DHTread(){ //Sensor DHT22 capta los datos  
    uint8 IState;
    int i;
    IState=CyEnterCriticalSection();  //Desactiva las interrupciones y devuelve un valor de estado de interrupción
    uint8 bytes[5]; 
    uint8 cnt = 7; 
    uint8 idx = 0; 
    int   calc=0; 
    int   timeout=0; 
    
    /* 
    //VARIABLES VERIFICACION DEL CHECKSUM
        char outp[16];
        char outp1[16];
        uint8 checksum=0;
    */    
    
    
    //Ponemos a 0 todos los Bytes
    for (i=0; i< 5; i++)  
        bytes[i] = 0; 
    
    //SE REALIZA EL "TIMING" DEL SENSOR DHT22
    //Comienza en 1 (alto)
    //Se pone a 0 (bajo) durante 19 microsegundos
    DHT_Write(0u); 
    CyDelay(19); 
    //Se pone a 1 (alto)
    DHT_Write(1u); 
    //Estando a 1 (alto) el propio sensor DHT22 debe ponerlo a 0 (bajo)
    while(DHT_Read()==1) 
    { 
       timeout++; 
       if(timeout>500) //Si pasa mas de este tiempo en 1 (alto) se procede a anunciar un error
           goto r99;  //Error de funcionamiento del sensor DHT22 
    } 
    //Si no se detecta error, el sensor habrá puesto el valor a 0 (bajo)
    while(DHT_Read()==0) 
    {         
       timeout++; 
       if(timeout>500) //Si pasa mas de este tiempo en 0 (bajo) se procede a anunciar un error
           goto r99;  //Error de funcionamiento del sensor DHT22 
    } 
    //Si no se detecta error, el sensor habrá puesto el valor a 1 (alto)
    calc=timeout; 
    timeout=0; 
    //Estando el valor a 1 (alto) el sensor capta los datos de humedad y temperatura y lo almacena en Bytes
    while(DHT_Read()==1); 
    for (i=0; i<40; i++) 
	  { 
       timeout=0; 
       while(DHT_Read()==0); 
       while(DHT_Read()==1) 
           timeout++; 
       //Punto de adquisición de datos
       if ((timeout) > (calc/2)) 
           bytes[idx] |= (1 << cnt); 
       if (cnt == 0)   // Proximo Byte 
   	    { 
   		    cnt = 7;    // restart at MSB 
   		    idx++;      // Siguiente Byte 
   	    } 
   	    else cnt--; //Se disminuye "cnt" hasta que sea 0 y entre en el "if" anterior para el proximo byte
      } 
   //Se asignan los bytes que pertenecen a humedad y a temperatura
    humidity1    = bytes[0]; 
    humidity2    = bytes[1]; 
    humidity     = humidity1 * 256 + humidity2; //Humedad final
    temperature1 = bytes[2]; 
    temperature2 = bytes[3]; 
    temperature  = temperature1 * 256 + temperature2; //Temperatura final
    
    CyExitCriticalSection(IState);  //Restaura el estado de interrupción.
    
    
    /*
    //VERIFICACION DEL CHECKSUM
    
    checksum = bytes[0] + bytes[1] + bytes[2] + bytes[3]; 

    if ((checksum == bytes[4]) && (bytes[4] != 0))
        {
        //return 0;
        sprintf(outp,"Bien");
        setCursor(0,0);
        LCD_print(outp);
        CyDelay(3000);
        //Checksum verificado, muestra un 1
        }
        else{
        //return 99;
        sprintf(outp1,"Mal");
        setCursor(0,0);
        LCD_print(outp1);
        CyDelay(3000);
        //Checksum no verificado, muestra un 0
        }
    */   
    
    CyDelay(1); 
    return 0;
        
   r99:    //Goto label for error in DHT reading 
        
       //Si se produjera algun error las variables "humidity" y "temperature" almacenarian un valor de 99
       humidity    = 99;  
       temperature = 99;  
       CyExitCriticalSection(IState); //Restaura el estado de interrupción.
       return 99; 
}

int main(void)
{
    CyGlobalIntEnable; /* Enable global interrupts. */
    
    //Se definen los dos char que vamos a mostrar en la LCD para la humedad y temperatura
    char outputstring1[40]; //Humedad
    char outputstring2[40]; //Temperatura
    
    I2C_Start(); //Inicializamos I2C
    
    LiquidCrystal_I2C_init(Addr,16,2,0); //Se inicializa LCD (direccion, longitud de linea, nº lineas, tamaño char)
   
    begin(); //LCD
    

    
    CyBle_Start(BLECallBack);
    
    for(;;)
    { 
        CyBle_ProcessEvents();
        
        if(deviceConnected){
            
            DHTread(); //Se captan los datos de humedad y temperatura
        
            humidity = humidity / 10;
            //Se muestra la humedad
            sprintf(outputstring1,"Humidity=%.2i%%",humidity);
            setCursor(0,0); //Primera linea LCD
            LCD_print(outputstring1);
            //CyDelay(3000);
        
        
            temperature = temperature / 10;
            //Se muestra la temperatura
            sprintf(outputstring2,"Temperature=%.2ioC",temperature); 
            setCursor(0,1); //Segunda linea LCD
            LCD_print(outputstring2);
            //UART_UartPutString(outputstring);
            CyDelay(2000);  //Delay in milli seconds. 
        
            notificationHandle.attrHandle = CYBLE_DHT22_SENSOR_CHAR_HANDLE;
            notificationHandle.value.val = (uint8*) &humidity;
            notificationHandle.value.len = 2;
            //////////////////////////////////
            CyBle_GattsWriteAttributeValue(&notificationHandle,0,&connectionHandle, 0);
            //////////////////////////////////
            if(startNotification)
            {
            CyBle_GattsNotification(connectionHandle, &notificationHandle);
            }
        }
    }
}


/* [] END OF FILE */
